/*
 * This class implements a RMI bandwidth gauge.
 *
 */

package edu.cmu.cs.able.gauges.RMI;
import java.rmi.*;
import java.util.*;

import java.net.*;
import java.io.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;

public class RMIBWGauge extends RMIGauge {

    /** Creates new RMIBWGauge
     * @param gaugeID The ID of the gauge to create.
     * @param creatorID The ID of the gauge manager that is creating the gauge.
     * @param setupParams The value of the setup params.
     */
	public RMIBWGauge (GaugeID gaugeID, String creatorID, StringPairVector setupParams)
	{
		super(gaugeID, creatorID, setupParams);
	}

    public void reportNewValue (){
        gaugeBus.reportValue(gaugeID, "Bandwidth", "", "10");
    }

    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     */    
	public static void main(String args[])
	{

		String gaugeName = "RMIBWGaugeName";   
		String gaugeType = "edu.cmu.cs.able.gauges.RMI.RMIBWGauge";   
		String modelName = "RMIBWGaugeModelName";   
		String modelType = "RMIBWGaugeModelType";   				    
        String creatorID = "";

        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        setupParams.addElement("Frequency", "500");
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                } 
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-rmib")) {
            	++i;
            }
            else {
                ok = false;
            }            
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0);
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;

   		System.out.println("RMIBWGauge started");
   		System.out.flush();

   		RMIBWGauge theBWGauge = new RMIBWGauge(gid, creatorID, setupParams);
   		
   		for (int i = 0; i < 1000; i++) {
   			theBWGauge.reportNewValue();
   			System.err.println("reported");
   			try {
   				Thread.sleep(1000);
   			}
   			catch (Exception e) 
   			{
   				e.printStackTrace();
   			}
   		}
	}
    
    private static void printUsageMessage () {
        System.out.println ("Usage: RMIBWGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -rmib <rmibus>");
    }
    
}
